import bs
import bsUtils
import random

class Background(bs.Actor):
    "Simple Fading Background Node-Group"

    def __init__(self,activity,retain,fadeTime=500,startFaded = False,showLogo=False):
        bs.Actor.__init__(self,activity,retain)
        self._dying = False
        self.fadeTime=fadeTime
        self.node = bs.newNode("image",
                               owner=activity,
                               delegate=self,
                               attrs={"fillScreen":True,
                                      "texture":bs.getTexture('bg'),
                                      "tiltTranslate":-0.3,
                                      "hasAlphaChannel":False,
                                      "color":(1,1,1)},
                               name='Background',
                               isCrossFade=True)

        if not startFaded:
            bsUtils.animate(self.node,"opacity",{0:0, self.fadeTime:1},loop=False)

        if showLogo:
            logoTexture = bs.getTexture("logo")
            logoModel = bs.getModel("logo")
            logoModelTransparent = bs.getModel("logoTransparent")
            self.c = bs.newNode("combine",owner=self.node,attrs={"size":2},name='Logo Scale Driver')
            for attr in ['input0','input1']:
                bsUtils.animate(self.c,attr,{0:0.693, 50:0.7, 500:0.693},loop=True)
            self.logo = bs.newNode("image",
                                   owner=self.node,
                                   attrs={"texture":logoTexture,
                                          "modelOpaque":logoModel,
                                          "modelTransparent":logoModelTransparent,
                                          "scale":(1,0.7),
                                          "color":(0.15,0.15,0.15),
                                          "translate":(0,0),
                                          "tiltTranslate":-0.05,
                                          "absoluteScale":False},name="Logo")
            bs.connectAttr(self.c,"output",self.logo,"scale")
            bs.connectAttr(self.node,"opacity",self.logo,"opacity")

            c = bs.newNode("combine",owner=self.node,name='Logo Translate Driver')
            bs.connectAttr(c,'output',self.logo,'translate')

            # gen some random keys for that stop-motion-y look
            keys = {}
            time = 0
            for i in range(10):
                keys[time] = (random.random()-0.5)*0.0015
                time += random.random() * 100
            bsUtils.animate(c,"input0",keys,loop=True)
            keys = {}
            time = 0
            for i in range(10):
                keys[time] = (random.random()-0.5)*0.0015 + 0.05
                time += random.random() * 100
            bsUtils.animate(c,"input1",keys,loop=True)


    def handleMessage(self,m):
        if isinstance(m,bs.DieMessage):
            if not self._dying and bs.nodeExists(self.node):
                # if m.immediate:
                #     bs.deleteNode(self.node)
                # else:
                self._dying = True
                bsUtils.animate(self.node,"opacity",{0:1, self.fadeTime:0},loop=False)
                bs.gameTimer(self.fadeTime+100,bs.WeakCall(bs.deleteNode,self.node))

        else: bs.Actor.handleMessage(self,m)


class Text(bs.Actor):
    """ just a text node with a few tricks """

    def __init__(self,activity,retain,text,position=(0,0),hAlign='left',vAlign='none',color=(1,1,1,1),transition=None,transitionDelay=0,flash=False,
                 vAttach='center',hAttach='center',scale=1.0,transitionOutDelay=None,maxWidth=None):
        bs.Actor.__init__(self,activity,retain)
        self.node = bs.newNode('text',
                               owner=activity,
                               attrs={'text':text,
                                      'color':color,
                                      'translate':position,
                                      'hAlign':hAlign,
                                      'vAlign':vAlign,
                                      'hAttach':hAttach,
                                      'vAttach':vAttach,
                                      'maxWidth':0.0 if maxWidth is None else maxWidth,
                                      'scale':[scale]},
                               name="Announce Text: '"+text+"'",
                               delegate=self)

        if transition == 'fadeIn':
            if flash: raise Exception("fixme: flash and fade-in currently cant both be on")
            c = bs.newNode('combine',owner=self.node,attrs={'input0':color[0],'input1':color[1],'input2':color[2]})
            bs.connectAttr(c,'output',self.node,'color')
            keys = {transitionDelay:0,transitionDelay+500:color[3]}
            if transitionOutDelay is not None:
                keys[transitionDelay+transitionOutDelay] = color[3]
                keys[transitionDelay+transitionOutDelay+500] = 0
            bsUtils.animate(c,"input3",keys)

        if flash:
            c = bs.newNode('combine',owner=self.node)
            bs.connectAttr(c,'output',self.node,'color')
            bsUtils.animate(c,"input0",{0:color[0], 150:0.3},loop=True)
            bsUtils.animate(c,"input1",{0:color[1], 150:0.1},loop=True)
            bs.setAttr(c,'input2',color[2])
            bs.setAttr(c,'input3',color[3])

        c = self.translateCombine = bs.newNode("combine",owner=self.node)
        bs.connectAttr(c,'output',self.node,'translate')
        if transition == 'inRight':
            keys = {transitionDelay:position[0]+1300,transitionDelay+200:position[0]}
            bsUtils.animate(c,"input0",keys)
            bs.setAttr(c,"input1",position[1])
        elif transition == 'inLeft':
            keys = {transitionDelay:position[0]-1300,transitionDelay+200:position[0]}
            if transitionOutDelay is not None:
                keys[transitionDelay+transitionOutDelay] = position[0]
                keys[transitionDelay+transitionOutDelay+200] = position[0]-1300
            bsUtils.animate(c,"input0",keys)
            bs.setAttr(c,"input1",position[1])
        elif transition == 'inBottomSlow':
            keys = {transitionDelay:-50,transitionDelay+1000:position[1]}
            bs.setAttr(c,'input0',position[0])
            bsUtils.animate(c,"input1",keys)
        elif transition == 'inBottom':
            keys = {transitionDelay:-50,transitionDelay+200:position[1]}
            if transitionOutDelay is not None:
                keys[transitionDelay+transitionOutDelay] = position[1]
                keys[transitionDelay+transitionOutDelay+200] = -50
            bs.setAttr(c,'input0',position[0])
            bsUtils.animate(c,"input1",keys)
        elif transition == 'inTopSlow':
            keys = {transitionDelay:400,transitionDelay+3500:position[1]}
            bs.setAttr(c,'input0',position[0])
            bsUtils.animate(c,"input1",keys)
        else:
            bs.setAttr(c,'input0',position[0])
            bs.setAttr(c,"input1",position[1])

        # if we're transitioning out, die at the end of it
        if transitionOutDelay is not None:
            bs.gameTimer(transitionDelay+transitionOutDelay+1000,bs.WeakCall(self.handleMessage,bs.DieMessage()))

    def handleMessage(self,m):
        if isinstance(m,bs.DieMessage):
            bs.deleteNode(self.node,quiet=True)
        else:
            bs.Actor.handleMessage(self,m)

class Image(bs.Actor):

    """ just an image node with a few tricks """
    def __init__(self,activity,retain,texture,position=(0,0),transition=None,transitionDelay=0,attach='center',
                 color=(1,1,1,1),scale=(100,100),transitionOutDelay=None,modelOpaque=None,modelTransparent=None):
        bs.Actor.__init__(self,activity,retain)

        # if they provided a dict as texture, assume its an icon.. otherwise its just a texture value itself
        if type(texture) == dict:
            tintColor = texture['tintColor']
            tint2Color = texture['tint2Color']
            tintTexture = texture['tintTexture']
            texture = texture['texture']
        else:
            tintColor=(1,1,1)
            tint2Color=None
            tintTexture = -1
        

        self.node = bs.newNode('image',
                               owner=activity,
                               attrs={"texture":texture,
                                      "tintColor":tintColor,
                                      "tintTexture":tintTexture,
                                      "translate":position,
                                      "scale":scale,
                                      "color":color[:3],
                                      "opacity":color[3],
                                      "absoluteScale":True,
                                      "attach":attach},
                               delegate=self)

        if modelOpaque is not None: bs.setAttr(self.node,'modelOpaque',modelOpaque)
        if modelTransparent is not None: bs.setAttr(self.node,'modelTransparent',modelTransparent)
        if tint2Color is not None: bs.setAttr(self.node,'tint2Color',tint2Color)

        if transition == 'fadeIn':
            keys = {transitionDelay:0,transitionDelay+500:color[3]}
            if transitionOutDelay is not None:
                keys[transitionDelay+transitionOutDelay] = color[3]
                keys[transitionDelay+transitionOutDelay+500] = 0
            bsUtils.animate(self.node,"opacity",keys)

        c = self.translateCombine = bs.newNode("combine",owner=self.node)
        bs.connectAttr(c,'output',self.node,'translate')
        if transition == 'inRight':
            keys = {transitionDelay:position[0]+1200,transitionDelay+200:position[0]}
            bsUtils.animate(c,"input0",keys)
            bs.setAttr(c,"input1",position[1])
        elif transition == 'inLeft':
            keys = {transitionDelay:position[0]-1200,transitionDelay+200:position[0]}
            if transitionOutDelay is not None:
                keys[transitionDelay+transitionOutDelay] = position[0]
                keys[transitionDelay+transitionOutDelay+200] = -position[0]-1200
            bsUtils.animate(c,"input0",keys)
            bs.setAttr(c,"input1",position[1])
        elif transition == 'inBottomSlow':
            keys = {transitionDelay:-400,transitionDelay+3500:position[1]}
            bs.setAttr(c,'input0',position[0])
            bsUtils.animate(c,"input1",keys)
        elif transition == 'inBottom':
            keys = {transitionDelay:-400,transitionDelay+200:position[1]}
            if transitionOutDelay is not None:
                keys[transitionDelay+transitionOutDelay] = position[1]
                keys[transitionDelay+transitionOutDelay+200] = -400
            bs.setAttr(c,'input0',position[0])
            bsUtils.animate(c,"input1",keys)
        elif transition == 'inTopSlow':
            keys = {transitionDelay:400,transitionDelay+3500:position[1]}
            bs.setAttr(c,'input0',position[0])
            bsUtils.animate(c,"input1",keys)
        else:
            bs.setAttr(c,'input0',position[0])
            bs.setAttr(c,"input1",position[1])

        # if we're transitioning out, die at the end of it
        if transitionOutDelay is not None:
            bs.gameTimer(transitionDelay+transitionOutDelay+1000,bs.WeakCall(self.handleMessage,bs.DieMessage()))

    def handleMessage(self,m):
        if isinstance(m,bs.DieMessage):
            bs.deleteNode(self.node,quiet=True)
        else:
            bs.Actor.handleMessage(self,m)


class AchievementText(bs.Actor):
    """ a bit of text that pops up above events to note an achievement """
    def __init__(self,activity,retain,text,position=(0,0,0),color=(1,1,1,1),jitter=0.5,offset=(0,0,0),scale=1.0):
        bs.Actor.__init__(self,activity,retain)
        # a bit of random jitter
        pos = (position[0]+offset[0]+jitter*(0.5-random.random()),
               position[1]+offset[0]+jitter*(0.5-random.random()),
               position[2]+offset[0]+jitter*(0.5-random.random()))
        self.node = bs.newNode('text',
                               owner=activity,
                               attrs={"text":text,
                                      "inWorld":True,
                                      "hAlign":'center'},
                               name='Achievement Text',
                               delegate=self)

        lifespan = 1500.0

        # scale up
        #c = bs.newNode("combine",owner=self.node)
        #bs.connectAttr(c,'output',self.node,'scale')
        #for i in range(3):
        bsUtils.animate(self.node,'scale',{0:0.0,
                                          lifespan*0.11:0.020*0.7*scale,
                                          lifespan*0.16:0.013*0.7*scale,
                                          lifespan*0.25:0.014*0.7*scale})

        # translate upward
        self._tcombine = bs.newNode("combine",owner=self.node,attrs={"input0":pos[0],"input2":pos[2]})
        bs.connectAttr(self._tcombine,'output',self.node,'translate')
        bsUtils.animate(self._tcombine,"input1",{0:pos[1]+1.5,lifespan:pos[1]+2.0})

        # fade our opacity in/out
        self._combine = bs.newNode("combine",owner=self.node,attrs={"input0":color[0],"input1":color[1],"input2":color[2]})
        bs.connectAttr(self._combine,'output',self.node,'color')
        for i in range(4):
            #bsUtils.animate(self._combine,"input"+str(i),{0:0,0.1*lifespan:color[i],0.7*lifespan:color[i],lifespan:0})
            bsUtils.animate(self._combine,"input"+str(i),{0.13*lifespan:color[i],0.18*lifespan:4.0*color[i],0.22*lifespan:color[i]})
        bsUtils.animate(self._combine,"input3",{0:0,0.1*lifespan:color[3],
                                                #0.2*lifespan:color[3],0.24*lifespan:color[3]*3,0.28*lifespan:color[3],
                                                0.7*lifespan:color[3],lifespan:0})

        # kill ourself 
        self._dieTimer = bs.Timer(int(lifespan),bs.WeakCall(self.handleMessage,bs.DieMessage()))

    def handleMessage(self,m):
        if isinstance(m,bs.DieMessage):
            bs.deleteNode(self.node)
        else: bs.Actor.handleMessage(self,m)
            
class TipsText(bs.Actor):
    """ a bit of text that shows various messages; good for helpful-tips kinda things """
    def __init__(self,activity,retain,offsY=100):
        bs.Actor.__init__(self,activity,retain)
        self._tipScale = 0.8
        self._tipTitleScale = 1.1
        self._offsY = offsY
        self.node = bs.newNode('text',
                               owner=activity,
                               attrs={"text":'',
                                      "scale":[self._tipScale],
                                      "hAlign":'left',
                                      "vAlign":'center',
                                      "vAttach":'bottom'},
                               name='Info Text',
                               delegate=self)
        self.titleNode = bs.newNode('text',
                               owner=activity,
                               attrs={'text':bs.getResource('tipText')+':',
                                      'scale':[self._tipTitleScale],
                                      'maxWidth':122,
                                      'hAlign':'right',
                                      'vAlign':'center',
                                      'vAttach':'bottom'},
                               name='Info Text Title',
                               delegate=self)

        self._messageDuration = 10000
        self._messageSpacing = 3000

        self._changeTimer = bs.Timer(self._messageDuration+self._messageSpacing,
                                          bs.WeakCall(self.changePhrase),repeat=True)
        self._combine = bs.newNode("combine",owner=self.node,attrs={'input0':1.0,'input1':0.8,'input2':1.0})
        bs.connectAttr(self._combine,'output',self.node,'color')
        bs.connectAttr(self._combine,'output',self.titleNode,'color')
        self.changePhrase()

    def changePhrase(self):
        nextTip = bs.translate('tips',bs.getNextTip())
        s = self._messageSpacing
        if bs.nodeExists(self.node):
            tipWidth = self._tipScale * bs.getStringWidth(nextTip)
            if tipWidth > 850: tipExtraScale = 850.0/tipWidth
            else: tipExtraScale = 1.0
            bs.setAttr(self.node,'scale',[self._tipScale*tipExtraScale])
            bs.setAttr(self.node,'translate',(-0.5*(tipWidth*tipExtraScale)+55+15,self._offsY))
            bs.setAttr(self.titleNode,'translate',(-0.5*(tipWidth*tipExtraScale)+55,self._offsY+3))
        keys = {s:0,s+1000:1.0,s+self._messageDuration-1000:1.0,s+self._messageDuration:0.0}
        
        bsUtils.animate(self._combine,"input3",dict([[k,v*0.5] for k,v in keys.items()]))
        bs.setAttr(self.node,'text',nextTip)

    def handleMessage(self,m):
        if isinstance(m,bs.DieMessage):
            bs.deleteNode(self.node,quiet=True)
            bs.deleteNode(self.titleNode,quiet=True)
        else: bs.Actor.handleMessage(self,m)

class ZoomText(bs.Actor):
    "Big Zooming Text Node-Group"

    def __init__(self,activity,retain,text,position=(0,0),shiftPosition=None,shiftDelay=None,lifespan=None,flash=True,trail=True,hAlign="center",
                 color=(0.9,0.4,0.0),jitter=0.0,trailColor=(1.0,0.35,0.1,0.0),scale=1.0,projectScale=1.0,tiltTranslate=0.0,maxWidth=None):
        bs.Actor.__init__(self,activity,retain)

        self._dying = False

        positionAdjusted = (position[0],position[1]-100)
        if shiftDelay == None: shiftDelay = 2500
        if shiftDelay < 0: raise Exception("negative shift delay")

        self._projectScale = projectScale
        self.node = bs.newNode("text",
                               owner=activity,
                               attrs={'translate':positionAdjusted,
                                      'big':True,
                                      'text':text,
                                      'trail':trail,
                                      'scale':[scale],
                                      'maxWidth':maxWidth if maxWidth is not None else 0.0,
                                      'tiltTranslate':tiltTranslate,
                                      'hAlign':hAlign,
                                      'vAlign':'center'},
                               name='Zoom Text: \''+text+'\'',
                               delegate=self)
            
        # if they want jitter, animate its position slightly...
        if jitter > 0.0:
            self._jitter(positionAdjusted,jitter*scale)

        # if they want shifting, move to the shift position and then resume jittering
        if shiftPosition != None:
            positionAdjusted2 = (shiftPosition[0],shiftPosition[1]-100)
            bs.gameTimer(shiftDelay,bs.WeakCall(self._shift,positionAdjusted,positionAdjusted2))
            if jitter > 0.0:
                bs.gameTimer(shiftDelay+250,bs.WeakCall(self._jitter,positionAdjusted2,jitter*scale))
                #bs.gameTimer(shiftDelay+250,(self._jitter,(positionAdjusted2,jitter)))
         

        colorCombine = bs.newNode("combine",
                                  owner=self.node,
                                  attrs={"input2":color[2],
                                         "input3":1.0})
        bs.connectAttr(colorCombine,"output",self.node,"color")

        if trail:
            trailColor = bs.newNode("combine",
                                    owner=self.node,
                                    attrs={"input0":trailColor[0],
                                           "input1":trailColor[1],
                                           "input2":trailColor[2],
                                           "input3":trailColor[3]})
            bs.connectAttr(trailColor,"output",self.node,"trailColor")
            foo = 0.85
            # bsUtils.animate(self.node,"trailProjectScale",{0:0, int(foo*201):0.6, int(foo*347):0.8,
            #                                                int(foo*478):0.9, int(foo*595):0.93, int(foo*748):0.95, int(foo*941):0.95})
            bsUtils.animate(self.node,"trailProjectScale",{0:0*projectScale, int(foo*201):0.6*projectScale, int(foo*347):0.8*projectScale,
                                                           int(foo*478):0.9*projectScale, int(foo*595):0.93*projectScale, int(foo*748):0.95*projectScale, int(foo*941):0.95*projectScale})
        if flash:
            bsUtils.animate(colorCombine,"input0",{0:color[0], 150:0.2+2.0*color[0]},loop=True)
            bsUtils.animate(colorCombine,"input1",{0:color[1], 150:0.15+2.0*color[1]},loop=True)
            bsUtils.animate(colorCombine,"input2",{0:color[2], 150:0.1+2.0*color[2]},loop=True)
        else:
            bs.setAttr(colorCombine,'input0',color[0])
            bs.setAttr(colorCombine,'input1',color[1])
            
        # bsUtils.animate(self.node,"projectScale",{0:0, 270:1.05, 300:1})
        bsUtils.animate(self.node,"projectScale",{0:0, 270:1.05*projectScale, 300:1*projectScale})

        # if they give us a lifespan, kill ourself down the line
        if lifespan != None:
            bs.gameTimer(lifespan,bs.WeakCall(self.handleMessage,bs.DieMessage()))

    def handleMessage(self,m):
        if isinstance(m,bs.DieMessage):
            if not self._dying and bs.nodeExists(self.node):
                # if m.immediate:
                #     bs.deleteNode(self.node)
                #if not self._dying:
                self._dying = True
                bsUtils.animate(self.node,"projectScale",{0:1*self._projectScale, 600:1.2*self._projectScale})
                bsUtils.animate(self.node,"opacity",{0:1, 300:0})
                bsUtils.animate(self.node,"trailOpacity",{0:1, 600:0})
                bs.gameTimer(700,bs.WeakCall(bs.deleteNode,self.node))
        else:
            bs.Actor.handleMessage(self,m)

    def _jitter(self,position,jitterAmount):
        if not bs.nodeExists(self.node): return
        c = bs.newNode("combine",owner=self.node)
        bs.connectAttr(c,'output',self.node,'translate')

        for index,attr in enumerate(['input0','input1']):
            keys = {}
            time = 0
            # gen some random keys for that stop-motion-y look
            for i in range(10):
                keys[time] = position[index] + (random.random()-0.5)*jitterAmount*1.6
                time += random.random() * 100
            bsUtils.animate(c,attr,keys,loop=True)

    def _shift(self,position1,position2):
        if not bs.nodeExists(self.node): return
        c = bs.newNode("combine",owner=self.node)
        bs.connectAttr(c,'output',self.node,'translate')
        bsUtils.animate(c,'input0',{0:position1[0],250:position2[0]})
        bsUtils.animate(c,'input1',{0:position1[1],250:position2[1]})


